/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006 by Kevin Gilbert                                   *
 *   kev.gilbert@cdu.edu.au                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include <qvbox.h>

#include <kapplication.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <kdebug.h>
#include <klocale.h>

#include "gui_rpc_client.h"
#include "statisticscontroldata.h"
#include "statisticscontrolview.h"

//	constructor
//	===========

StatisticsControlView::StatisticsControlView( const CPtr client, QWidget* parent, const char* name )
: QScrollView( parent, name ),
  m_client( client )
{	KConfig* config = kapp->config( );
	config->setGroup( "StatisticsGraphControl_" + m_client->clientName );

	QString currentProject = config->readEntry( "currentProject", QString::null );
	ushort  noOfProjects   = config->readNumEntry( "noOfProjects", 0 );
	QString text;

    for ( ushort i = 0; i < noOfProjects; i++ )
	{	QString key = QString( "projectData_%1" ).arg( i );
		m_projectsData.append( new StatisticsControlData( config->readEntry( key, QString::null )));
	}

	createLayout( currentProject );
	finaliseInitialisation( );
}

//	compareProjects
//	===============

int StatisticsControlView::compareProjects( const void* project1, const void* project2 )
{	return (*SCDPPtr( project1 ))->compare( *SCDPPtr( project2 ));
}

//	createLayout
//	============

void StatisticsControlView::createLayout( const QString& currentProject )
{	byte spacing = fontMetrics( ).width( "XX" );

	QVBox* vBox = new QVBox( viewport( ));
	vBox->setEraseColor( viewport( )->eraseColor( ));
	vBox->setMargin( spacing / 2 );
	vBox->setSpacing( spacing );

	addChild( vBox );
	PROJECTS projects;
	int statr = m_client->rpcClient.get_project_status( projects );

	if( statr )
	{	m_client->reason.sprintf( i18n( "get_project_status returned error %d" ), statr );
		return;
	}

	ushort size = projects.projects.size( );

	if( !size )
		return;

	QWidget* spacer1 = new QWidget( vBox );
	spacer1->setEraseColor( KGlobalSettings::buttonBackground( ));

	m_projectsComboBox  = new KComboBox( false, vBox, "projects combo box" );
	connect( m_projectsComboBox, SIGNAL( activated( const QString& )), SLOT( slotProjectActivated( const QString& )));

	QWidget* spacer2 = new QWidget( vBox );
	spacer2->setEraseColor( KGlobalSettings::buttonBackground( ));
	vBox->setStretchFactor( spacer2, 10 );

	m_graphTypeComboBox  = new KComboBox( false, vBox, "projects combo box" );
	connect( m_graphTypeComboBox, SIGNAL( activated( int )), SLOT( slotGraphActivated( int )));

	ushort currentIndex = 0;

	for( ushort i = 0; i <= size; i++ )
	{	QString projectName  = (i == size) ? StatisticsControlData::StatsStackedLit : QString( projects.projects[ i ]->project_name );
		SCDPtr projectData;

    	for( projectData = m_projectsData.first( ); projectData != NULL; projectData = m_projectsData.next( ))
			if( projectName == projectData->projectName( ))
				break;

		if( projectData == NULL )
		{	projectData = new StatisticsControlData( UserTotal, projectName );
			m_projectsData.append( projectData );
		}

		m_projectsComboBox->insertItem( projectName );
		projectData->guiRPCIndex( i );
		projectData->isKnown( true );

		if( i == size )
			projectData->stacked( true );

		if( projectName == currentProject )
			currentIndex = i;
	}

	m_projectsComboBox->setCurrentItem( currentIndex );

	m_graphTypeComboBox->insertItem( i18n( "Host Average" ));
	m_graphTypeComboBox->insertItem( i18n( "Host Total"   ));
	m_graphTypeComboBox->insertItem( i18n( "User Average" ));
	m_graphTypeComboBox->insertItem( i18n( "User Total"   ));

	switch( findCurrentProject( )->graphType( ))
	{	case HostAvg:   m_graphTypeComboBox->setCurrentItem( 0 ); break;
		case HostTotal: m_graphTypeComboBox->setCurrentItem( 1 ); break;
		case UserAvg:   m_graphTypeComboBox->setCurrentItem( 2 ); break;
		case UserTotal: m_graphTypeComboBox->setCurrentItem( 3 ); break;
	}
}

//	finaliseInitialisation
//	======================

void StatisticsControlView::finaliseInitialisation( )
{	ushort  noOfProjects = m_projectsData.count( );
	SCDPPtr sortData     = new SCDPtr[ noOfProjects ];
	ushort  i;
	SCDPtr  projectData;

    for( i = 0, projectData = m_projectsData.first( ); projectData != NULL; i++, projectData = m_projectsData.next( ))
		sortData[ i ] = projectData;

	::qsort( sortData, noOfProjects, sizeof( SCDPtr ), compareProjects );
	m_projectsData.clear( );

	for( i = 0; i < noOfProjects; i++ )
		if( sortData[ i ]->isKnown( ))
			m_projectsData.append( sortData[ i ]);
		else
			delete sortData[ i ];

	delete [] sortData;
	m_projectsData.setAutoDelete( true );
}

//	findCurrentProject
//	==================

SCDPtr StatisticsControlView::findCurrentProject( )
{	QString projectName = m_projectsComboBox->currentText( );
	SCDPtr  projectData;

    for( projectData = m_projectsData.first( ); projectData != NULL; projectData = m_projectsData.next( ))
		if( projectData->projectName( ) == projectName )
			return projectData;

	fprintf( stderr, "internal error in StatisticsControlView::findCurrentProject - cannot find project \"%s\"\n", projectName.ascii( ));
	abort( );
	return NULL;
}

//	hostAveragelActivated
//	=====================

int kmg1 = 1;

void StatisticsControlView::hostAveragelActivated( )
{	SCDPtr projectData = findCurrentProject( );
	projectData->graphType( HostAvg );
	emit( hostAverageActivated( projectData->projectName( )));
}

//	hostTotalActivated
//	==================

void StatisticsControlView::hostTotalActivated( )
{	SCDPtr projectData = findCurrentProject( );
	projectData->graphType( HostTotal );
	emit( hostTotalActivated( projectData->projectName( )));
}

//	saveSettings
//	============

void StatisticsControlView::saveSettings( )
{	KConfig* config = kapp->config( );
	config->setGroup( "StatisticsGraphControl_" + m_client->clientName );
	config->writeEntry( "currentProject", findCurrentProject( )->projectName( ));
	config->writeEntry( "noOfProjects",   m_projectsData.count( ));

	ushort  i;
	SCDPtr  projectData;
	QString text;

    for( i = 0, projectData = m_projectsData.first( ); projectData != NULL; i++, projectData = m_projectsData.next( ))
		if( projectData->isKnown( ))
			config->writeEntry( QString( "projectData_%1" ).arg( i ), projectData->toString( ));
}

//	slotGraphActivated
//	==================

void StatisticsControlView::slotGraphActivated( int index )
{	switch( index )
	{	case 0: hostAveragelActivated( ); break;
		case 1: hostTotalActivated( );    break;
		case 2: userAverageActivated( );  break;
		case 3: userTotalActivated( );    break;
	}
}

//	slotProjectActivated
//	====================

void StatisticsControlView::slotProjectActivated( const QString& /* projectName */)
{	switch( findCurrentProject( )->graphType( ))
	{	case HostAvg:   m_graphTypeComboBox->setCurrentItem( 0 ); hostAveragelActivated( ); break;
		case HostTotal: m_graphTypeComboBox->setCurrentItem( 1 ); hostTotalActivated( );    break;
		case UserAvg:   m_graphTypeComboBox->setCurrentItem( 2 ); userAverageActivated( );  break;
		case UserTotal: m_graphTypeComboBox->setCurrentItem( 3 ); userTotalActivated( );    break;
	}
}

//	userAverageActivated
//	====================

void StatisticsControlView::userAverageActivated( )
{	SCDPtr projectData = findCurrentProject( );
	projectData->graphType( UserAvg );
	emit( userAverageActivated( projectData->projectName( )));
}

//	userTotalActivated
//	==================

void StatisticsControlView::userTotalActivated( )
{	SCDPtr projectData = findCurrentProject( );
	projectData->graphType( UserTotal );
	emit( userTotalActivated( projectData->projectName( )));
}
